
**********************************************************************
*
* Program Name: ARSYN8.ASM ( Arithmetic Synthesizer )
* Revision: 1.00
* Date: February 3,1993
*
* Written By: Mark Glenewinkel
*             Motorola CSIC Applications
*
* Assembled Under: P&E Microcomputer Systems IASM08
*
*       *********************************
*       *       Revision History        *
*       *********************************
*
*       Rev     0.50    12/15/93        M.A. McQuilken
*                       HC05 version to be translated to HC08 code
*
*       Rev     0.60    01/21/93        M.R. Glenewinkel
*                       Added more comments
*
*       Rev     1.00    02/18/93        M.R. Glenewinkel
*                       HC08 version
*
**********************************************************************
*
* Program Description:
*
*       This routine produces a sinusoid of a specified frequency at
*       the output of a Digital-to-Analog Converter (DAC) attached
*       to Port X (set by the user) of an HC08.
*
*       Basically, this method utilizes a standard DAC look-up table
*       consisting of (in this case) 256 phase values for a single
*       cycle of a sinewave. The position in this table is determined
*       by two bytes of data: the MSB which is the integer index into
*       the DAC look-up table, and the LSB which is the fractional
*       depth into the table. Since this fractional portion is not
*       used to directly address a sine sample directly, its effect
*       is made only when the continual accumulation of this fraction
*       causes an overflow into the integer portion.
*
*       We can determine, with the appropriate choice of fraction
*       and integer, the output frequency of the digitized sinewave
*       to a high degree of accuracy. The exact mathematical
*       relationship, for 256 phase values, is:
*
*       Accumulation Constant = 256 x Desired Frequency x Sample Time
*
*       The sample time, in this case, is 3.625 usec using an HC08
*       running with an 8MHz E clock. The sample period is calculated
*       by determining the number of instructions within the
*       generating loop (29 cyc) and multiplying the number by the
*       bus clock period. In this example an 8kHz sinewave is to be
*       sythesized. The accumulation constant will be:
*
*                              D = 7.424
*
*       Thus, in the source code, the memory locations Int_K and
*       Frac_K should contain $07 and $6C, respectively, upon entering
*       the ArithSyn routine.
*
********************************* CAUTION ****************************
* Please understand the impact that code and/or hardware changes may
* have on the desired, synthesized output frequency. Before making
* changes be certain that you understand the ramifications.
**********************************************************************
*
********************************* NOTE *******************************
* This code assumes that you've got an appropriate DAC on PortB (in
* this case, the testbed consisted of an AD557JN on PortB) and that
* you've made PortB an output port in your initialization code.
**********************************************************************
*
* TASK DATA:
*  Input Variables      Output Variables        Description
*  ----------------     -----------------       -------------
*  Frac_K                                       Enter with the
*                                                appropriate
*                                                fractional
*                                                accumulate value.
*  Int_K                                        Enter with appropriate
*                                                integer accumulation
*                                                value.
*
*
*
* LOCAL DATA:
*  Input Variables      Output Variables        Description
*  ----------------     -----------------       -------------
*  Cntr                 Cntr                    Determines length of
*                                                time the sinewave is
*                                                generated with a
*                                                maximum value of
*                                                approx. 4.5 msec.
*  AccumLSB             AccumLSB                Least significant
*                                                byte of 16-bit
*                                                phase accumulator.
*  AccumMSB             AccumMSB                Most significant
*                                                byte of 16-bit
*                                                phase accumulator.
*                                                This is the value
*                                                that is used to point
*                                                into the sine table.
*  ACCA                 ACCA                    Misc. computational
*                                                use.
*  X                    X                       Misc. computational
*                                                use.
*
**********************************************************************
*
* Register and Variable Equates
*
PortB           equ     $01
DAC             equ     PortB
*
**********************************************************************
*
* Memory
*
                ORG     $50
AccumLSB        RMB     1
AccumMSB        RMB     1
Frac_K          RMB     1
Int_K           RMB     1
*
**********************************************************************

                ORG     $6E00           ;beginning of program area
START           EQU     *

**********************************************************************

* Main Routine

* We must prepare the workspace, any nonzero variables could
* alter our process.

ArithSyn        clr     AccumLSB
                clr     AccumMSB


* At this point we're ready to actually begin the process of
* generating the sinewave. The accumulation is done with an 8-bit
* fraction and an 8-bit integer.
* So, basically, we're going to have the HC08 do a 16-bit addition:

SignalGen       lda     AccumLSB        ;3 - Get current LSB value
                                        ;  of the phase accum.
                add     Frac_K          ;3 - The fractional
                                        ;  constant is added to
                                        ;  the LSB of the phase
                                        ;  accumulator.
                sta     AccumLSB        ;3 - Make sure that
                                        ;  the updated value is
                                        ;  kept for the next time
                                        ;  through the loop.

* Here's the second half of our 16-bit addition. This will
* propagate any overflow from the 8-bit addition of the
* AccumLSB and Frac_K into AccumMSB:

                lda     AccumMSB        ;3 - Get current MSB value
                                        ;  of the phase accum.
                adc     Int_K           ;3 - The integer constant
                                        ;  is added to the MSB of
                                        ;  the phase accum.
                                        ;    Notice the addition
                                        ;  with carry.
                sta     AccumMSB        ;3 - Save for next time
                                        ;  through the loop.

* If we've made it here, then we are ready to turn our phase
* value into a sine wave at the output of the DAC:

                tax                     ;1 - ACCA contains the
                                        ;  integer portion of the
                                        ;  16-bit phase value.
                                        ;  We need to move it into
                                        ;  the X-reg to do a table
                                        ;  look-up.
                lda     SineTable,X     ;4 - Get the sine value
                sta     DAC             ;3 - Send it to the DAC
                                        ;  to create a real-world
                                        ;  signal.

                bra     SignalGen       ;5 - Branch to top of
                                        ;  signal generation
                                        ;  for an infinite loop

**********************************************************************

* Tables

SineTable       FCB     $80,$83,$86,$89,$8C,$90,$93,$96
                FCB     $99,$9C,$9F,$A2,$A5,$A8,$AB
                FCB     $AE,$B1,$B3,$B6,$B9,$BC,$BF,$C1
                FCB     $C4,$C7,$C9,$CC,$CE,$D1,$D3
                FCB     $D5,$D8,$DA,$DC,$DE,$E0,$E2,$E4
                FCB     $E6,$E8,$EA,$EB,$ED,$EF,$F0
                FCB     $F1,$F3,$F4,$F5,$F6,$F8,$F9,$FA
                FCB     $FA,$FB,$FC,$FD,$FD,$FE,$FE
                FCB     $FE,$FF,$FF,$FF,$FF,$FF,$FF,$FF
                FCB     $FE,$FE,$FE,$FD,$FD,$FC,$FB
                FCB     $FA,$FA,$F9,$F8,$F6,$F5,$F4,$F3
                FCB     $F1,$F0,$EF,$ED,$EB,$EA,$E8
                FCB     $E6,$E4,$E2,$E0,$DE,$DC,$DA,$D8
                FCB     $D5,$D3,$D1,$CE,$CC,$C9,$C7
                FCB     $C4,$C1,$BF,$BC,$B9,$B6,$B3,$B1
                FCB     $AE,$AB,$A8,$A5,$A2,$9F,$9C
                FCB     $99,$96,$93,$90,$8C,$89,$86,$83
                FCB     $80,$7D,$7A,$77,$74,$70,$6D
                FCB     $6A,$67,$64,$61,$5E,$5B,$58,$55
                FCB     $52,$4F,$4D,$4A,$47,$44,$41
                FCB     $3F,$3C,$39,$37,$34,$32,$2F,$2D
                FCB     $2B,$28,$26,$24,$22,$20,$1E
                FCB     $1C,$1A,$18,$16,$15,$13,$11,$10
                FCB     $0F,$0D,$0C,$0B,$0A,$08,$07
                FCB     $06,$06,$05,$04,$03,$03,$02,$02
                FCB     $02,$01,$01,$01,$01,$01,$01
                FCB     $01,$02,$02,$02,$03,$03,$04,$05
                FCB     $06,$06,$07,$08,$0A,$0B,$0C
                FCB     $0D,$0F,$10,$11,$13,$15,$16,$18
                FCB     $1A,$1C,$1E,$20,$22,$24,$26
                FCB     $28,$2B,$2D,$2F,$32,$34,$37,$39
                FCB     $3C,$3F,$41,$44,$47,$4A,$4D
                FCB     $4F,$52,$55,$58,$5B,$5E,$61,$64
                FCB     $67,$6A,$6D,$70,$74,$77,$7A,$7D


**********************************************************************

* Vector Setup

                ORG     $FFFE
                DW      START           ;set up reset vector

**********************************************************************



	 


